# -*- coding:Utf-8 -*-

#le pseudo commentaire ci-dessus permet que ce fichier
#sois écrit avec des caractères utf8 (éèà)

from Tkinter import Label, Frame, Tk
from time import localtime
from calendar import monthcalendar
import GDN

#importation du module sqlite3 sous le nom sqlite
#En cas de changement de version on change seulement le numéro dans cette ligne 
import sqlite3 as sqlite




class Agenda( Tk ): # Définition de la classe Agenda par dérivation de la classe Tk 
    def __init__( self ):
        lBind = Label.bind # On fabrique une fonction locale de la méthode bind de la classe Label
        lGrid = Label.grid 
                           # Ainsi, monLabel.grid( row = 1, column = 1 ) sera remplacé par
                           # lGrid( monLabel, row = 1, column = 1 )
                           # Ces fonctions local sont environ 11 fois plus rapide. 

        Tk.__init__( self )  #Appel du constructeur (__init__) de Tk() dans mon constructeur (polymorphisme).

        # Les deux variables qui suivent serviront à commander le changement de moi
        # self sinifie que timeInM et timeInA seront un attribue des instances d'Agenda 
        self.timeInM = self.timeInA = 0 # self.timeInM et self.timeInA indique la valeur zerro.

        self.libJDS  = ( "Lu", "Ma", "Me", "Je", "Ve", "Sa", "Di", ) # Création de deux tuples ( Les tuples sont immuables )
        self.libMOIS = ( None, "Janvier", "Fevrier", "Mars", "Avril",
                         "Mai", "Juin", "Juillet", "Aout",
                         "Septembre", "Octobre", "Novembre", "Decembre" )

        # On récupère la date et l'heure du jour.
        temps = localtime() # Création d'une instance localtime du module time

        self.today = ( temps[0], temps[1], temps[2], )

        self.annee  = temps[0]
        self.moi    = temps[1]
        calendrier  = monthcalendar( self.annee, self.moi )


        self.title("Mbg Agenda") # Tk.title est le titre de ma fenêtre par héritage Agenda.title le devient
        self.frmHt = Frame( self, bg = "white" )
        self.frmBs = Frame( self )

        self.labMoi   = Label(self.frmHt, text = self.libMOIS[ self.moi ],  bg = "white" )
        self.labAnnee = Label(self.frmHt, text = str( self.annee ),         bg = "white" )

        lab = None
        self.funcids = []
        for i in range( 6 ) :
            for j in range( 7 ) :
                lab = Label( self.frmBs, text = None,  foreground = "#000000", bg = "yellow",
                                    width = 3, height = 2, relief = "groove", anchor = "nw" )
                # bind rattache un événement à un widget et retourne le funcid
                # Il est primordiale de donner le funcid lors d'un appelle à unbind() !
                # Même si unbind() n'en a pas besoin pour détacher l'événement au widget
                # il en a besoin pour rendre la mémoire à l'OS.
                # widget.unbind( événement, funcid ) A faire :-D
                # widget.unbind( événement ) A ne pas faire :-(
                
                self.funcids.append(
                    [ lBind( lab, "<Button-1>", self.gestionDeNote    ),
                      lBind( lab, "<Enter>",    self.changeStyleLabel ),
                      lBind( lab, "<Leave>",    self.changeStyleLabel ), ]
                )

                lGrid( lab, row = i, column = j )

        self.allLabels = self.frmBs.grid_slaves() # Retourne tous les labels gridés dans l'ordre dé-chronologique.
        self.allLabels.reverse()


        # GDE (Gestionnaire D'Evenement)
        # Le premier argument passé à changeMoi est self le second est l'objet Event.
        # Pour plus de renseignements sur Event, dans un shell python saisir : 
        # import Tkinter 
        # help( Tkinter.Event )
        lBind( self.labMoi,   "<Enter>", self.changeMoi )
        lBind( self.labAnnee, "<Enter>", self.changeMoi )

        #GDP (Gestionnaire De Position)
        self.frmHt.grid(    row=0, column = 0, sticky='ew' ) # sticky='ew' Demande à la frame du haut de prendre toute la largeur.
        self.frmBs.grid(    row=1, column = 0 )
        self.labMoi.grid(   row=0, column = 0 )
        self.labAnnee.grid( row=0, column = 1 )


        self.affMoiApres()
        self.affMoiAvant()

        self.mainloop() # Démarre la fenêtre



    def changeMoi( self, event ) : # event contient un objet de type Event
        """
        Vérifie si la sourie est passée dans le label labMoi et labAnnee en moins de 750 ms
        Si c'est le cas Agenda affiche un autre moi selon le sens ou les labels ont été traversés.
        """
        if event.type == '7' : # Si la souris est entrée dans le widget
            if event.widget == self.labMoi : 
                self.timeInM = event.time
                if self.timeInM - self.timeInA < 750 :
                    self.affMoiAvant()

            elif event.widget == self.labAnnee:
                self.timeInA = event.time
                if self.timeInA - self.timeInM < 750 :
                    self.affMoiApres()


    def affMoiApres( self ):
        " Affiche le moi suivant."
        self.moi += 1
        if  self.moi >= 13 :
            self.moi = 1
            self.annee += 1

        self.updateAgenda()


    def affMoiAvant( self ):
        " Affiche le moi précèdent."
        self.moi -= 1
        if  self.moi <= 0 :
            self.moi = 12
            self.annee -= 1

        self.updateAgenda()


    def updateAgenda( self ):
        lBind   = Label.bind
        lUnbind = Label.unbind
        lGrid = Label.grid
        lConf = Label.configure
        calendrier = monthcalendar( self.annee, self.moi )
        nombreDeSemaine = len( calendrier )


        # Change le nom du moi
        self.labMoi.config(   text = self.libMOIS[ self.moi ] ) # Configure l'attribue text de labMoi.
        self.labAnnee.config( text = str( self.annee )        )

        con = sqlite.connect("BDD/notes.bdd") # Connexion à la bdd
        cur = con.cursor()
        cur.execute("""SELECT jour
                       FROM note
                       WHERE moi = ?
                       and annee = ? """, ( self.moi, self.annee, ) )

        # Compréhension de liste
        # evenementDuMoi est une liste contenant la première case de chacun des tuples retournés par cur.fetchall()
        evenementDuMoi = [ row[ 0 ] for row in cur.fetchall() ] #  [1,2,3] <== row[ 0 ] for row in ((1,'a','b',),(2,'a','b'),(3,'a','b'), )
        con.close()


        # master.grid_slaves() retourn tous les widgets gridés dans le master.
        labels = self.frmBs.grid_slaves()
        nmbDeLabelDisponible = len( labels )
        nmbDeSemaineDispo = nmbDeLabelDisponible /7


        j = numLabel = 0
        for semaine in calendrier :
            i = 0
            for jour in semaine :
                if j > nmbDeSemaineDispo - 1 : # S'il n'y avait pas assez de Labels, re-grider les labels effacés par remove()
                    # La méthode grid_remove efface le widget de l'écran, mais sa position est mémorisée. 
                    # On a seulement à appeler grid()
                    lGrid( self.allLabels[ numLabel ] )

            # Configurer les labels.
                if jour == 0 : #Si le jour n'existe pas. ex : lundi de la première semaine de mai 2009
                    text = self.libJDS[ i ]
                    self.allLabels[ numLabel ].configure( text = text, foreground = "#000000", bg = "yellow" )
                    lUnbind( self.allLabels[ numLabel ], "<Button-1>", self.funcids[ numLabel ][ 0 ] )
                    lUnbind( self.allLabels[ numLabel ], "<Enter>",    self.funcids[ numLabel ][ 1 ] )
                    lUnbind( self.allLabels[ numLabel ], "<Leave>",    self.funcids[ numLabel ][ 2 ] )
                    self.funcids[ numLabel ][ 0 ] = None
                    self.funcids[ numLabel ][ 1 ] = None
                    self.funcids[ numLabel ][ 2 ] = None


                else :
                    text =  self.libJDS[ i ] + "\n" + str( jour )
                    # Si le label correspond à la date du jour.
                    if jour == self.today[2] and self.moi == self.today[1] and self.annee == self.today[0] :
                        lConf( self.allLabels[ numLabel ], text = text, foreground = "red", bg = "yellow" )
                    else :
                        lConf( self.allLabels[ numLabel ], text = text, foreground = "#000000", bg = "yellow" )

                    if jour in evenementDuMoi : # Si le label correspond à une date en base de données.
                        lConf( self.allLabels[ numLabel ], bg="green" )

                    if self.funcids[ numLabel ][ 0 ] == None : # Si le label n'a plus d'évènement lui remettre. 
                        self.funcids[ numLabel ][ 0 ] = lBind( self.allLabels[ numLabel ], "<Button-1>", self.gestionDeNote    )
                        self.funcids[ numLabel ][ 1 ] = lBind( self.allLabels[ numLabel ], "<Enter>",    self.changeStyleLabel )
                        self.funcids[ numLabel ][ 2 ] = lBind( self.allLabels[ numLabel ], "<Leave>",    self.changeStyleLabel )

                numLabel += 1
                i += 1
            j += 1

        # Effacer les labels en trop.
        nmbLabelVoulu = nombreDeSemaine * 7
        while nmbDeLabelDisponible > nmbLabelVoulu :
            self.allLabels[ nmbDeLabelDisponible-1 ].grid_remove()
            nmbDeLabelDisponible -= 1


    def changeStyleLabel( self, event ) :
        if event.type == '7' : # Si la souris entre dans le label
            event.widget.configure( relief = "sunken" ) # Configure le relief du widget qui a provoqué l'évènement. 
            if event.widget['bg'] == "yellow" :
                event.widget.configure( bg = "#E0E000" )
        else :
            event.widget.configure( relief = "groove" )
            if event.widget['bg'] == "#E0E000" :
                event.widget.configure( bg = "yellow" )


    def gestionDeNote( self, event ) :
        GDN.GDN( event.widget, self.moi, self.annee )

if __name__ == "__main__" : # Si ce module n'est pas importé
    Agenda() #Instancie un Agenda.
